package cn.redcdn.network.tcp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import cn.redcdn.log.CustomLog;

enum ProcessorStatus {
	INIT, DESTROY
}

public class TCPProcessor {
	final static String tag = TCPProcessor.class.getName();
	final static int CMD_BUFFER = 1;

	private Socket mSocket;
	private String mServerAddress;
	private int mServerPort;
	private InputStream mInputStream;
	private OutputStream mOutputStream;
	private ProcessorStatus state = ProcessorStatus.DESTROY;

	private TCPReceiver tcpReceiver;

	private SocketConnectThread mConnectThread;
	private SocketReceiveThread mRecvThread;

	private HandlerThread mHandlerThread;
	// 子线程中的Handler实例。
	private Handler mSubThreadHandler;

	private static final int MSG_TIME_SEND = 1;
	private static final int MSG_SOCKET_CONNECT = 2;
	private static final int MSG_SOCKET_DISCONNECT = 3;
	private static final int MSG_SOCKET_CONNECTFAIL = 4;
	private static final int MSG_RECEIVE_DATA = 5;
	private static final int MSG_SEND_DATA = 6;

	private Handler mHandler = new Handler() {
		public void handleMessage(Message msg) {
			switch (msg.what) {
			// case MSG_TIME_SEND:
			// break;
			case MSG_SOCKET_CONNECT:
				// mSocketConnectState = STATE_CONNECTED;

				mRecvThread = new SocketReceiveThread();
				mRecvThread.start();
				break;
			case MSG_SOCKET_DISCONNECT:

				// mSocketConnectState = STATE_DISCONNECTED;
				destroy();
				reConnect();
				break;
			case MSG_SOCKET_CONNECTFAIL:
				// mSocketConnectState = STATE_DISCONNECTED;
				destroy();
				reConnect();
				break;
			case MSG_RECEIVE_DATA:
				tcpReceiver.process((String) msg.obj);
				break;
			default:
				break;
			}
		};
	};

	public TCPProcessor(TCPReceiver receiver) {
		tcpReceiver = receiver;
	}

	public int init(String addr, int port) {
		CustomLog.d(tag, "TCPProcessor init");

		if (ProcessorStatus.DESTROY != state) {
			CustomLog.e(tag, "TCPProcessor has initialized");
			return -1;
		}

		mServerAddress = addr;
		mServerPort = port;

		mConnectThread = new SocketConnectThread();
		mConnectThread.start();

		initHandlerThread();

		return 0;
	}

	private void reConnect() {
		mConnectThread = new SocketConnectThread();
		mConnectThread.start();

		initHandlerThread();
	}

	public void destroy() {
		CustomLog.d(tag, "TCPProcessor destroy");

		if (mHandlerThread != null) {
			mHandlerThread.quit();
			mHandlerThread = null;
			mSubThreadHandler = null;
		}
		
		if (ProcessorStatus.DESTROY == state) {
			CustomLog.e(tag, "TCPProcessor has destroyed");
			return;
		}

		state = ProcessorStatus.DESTROY;

		try {
			// 关闭输出流
			if (mOutputStream != null) {
				mOutputStream.close();
				mOutputStream = null;
			}
			// 关闭输入流
			if (mInputStream != null) {
				mInputStream.close();
				mInputStream = null;
			}
			// 关闭socket
			if (mSocket != null) {
				mSocket.close();
				mSocket = null;
			}
		} catch (IOException e1) {
			e1.printStackTrace();
			CustomLog.e(tag, "destroy fail " + e1.toString());
		}

		if (mRecvThread != null) {
			mRecvThread.threadExit();
			mRecvThread = null;
		}
	}

	public void sendMsg(String data) {
		if (ProcessorStatus.INIT != state) {
			CustomLog.e(tag, "TCPProcessor don't init");
			return;
		}
		if (mOutputStream == null) {
			CustomLog.e(tag, "TCPProcessor mOutputStream==NULL");
			return;
		}
		if (data.length() == 0) {
			CustomLog.e(tag, "TCPProcessor msg.length()==0");
			return;
		}

		Message msg = new Message();
		msg.what = MSG_SEND_DATA;
		msg.obj = data;
		mSubThreadHandler.sendMessage(msg);
	}

	private void writeMsg(String msg) {
		CustomLog.d(tag, "writeMsg msg=" + msg);
		if (msg.length() == 0 || mOutputStream == null)
			return;
		try {
			mOutputStream.write(msg.getBytes());// 发送
			mOutputStream.flush();
		} catch (Exception e) {
			e.printStackTrace();
			CustomLog.e(tag, "writeMsg fail " + e.toString());
		}

	}

	/**
	 * 判断是否断开连接，断开返回true,没有返回false
	 * 
	 * @param socket
	 * @return
	 */
	private Boolean isServerClose(Socket socket) {
		try {
			socket.sendUrgentData(0xFF);// 发送1个字节的紧急数据，默认情况下，服务器端没有开启紧急数据处理，不影响正常通信
			return false;
		} catch (Exception se) {
			return true;
		}
	}

	private void initHandlerThread() {
		// 创建HandlerThread实例
		mHandlerThread = new HandlerThread("handler_thread");
		// 开始运行线程
		mHandlerThread.start();
		// 获取HandlerThread线程中的Looper实例
		Looper loop = mHandlerThread.getLooper();
		// 创建Handler与该线程绑定。
		mSubThreadHandler = new Handler(loop) {
			public void handleMessage(Message msg) {
				CustomLog.d(tag, "mSubThreadHandler handleMessage thread:"
						+ Thread.currentThread());
				switch (msg.what) {
				case MSG_SEND_DATA:
					writeMsg((String) msg.obj);
					break;
				default:
					break;
				}
			};
		};
	}

	class SendHandler extends Handler {
		public SendHandler(Looper looper) {
			super(looper);
		}

		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case CMD_BUFFER:
				if (ProcessorStatus.INIT != state) {
					CustomLog.e(tag, "TCPProcessor don't init");
					return;
				}

				try {
					// 发送
					mOutputStream.write(((String) msg.obj).getBytes());
					mOutputStream.flush();
				} catch (Exception e) {
					e.printStackTrace();
					CustomLog.e(tag, "writeMsg fail " + e.toString());
				}

				break;

			default:
				break;
			}
		}
	};

	class SocketConnectThread extends Thread {
		public void run() {
			try {
				// 连接服务端，指定服务端ip地址和端口号。
				mSocket = new Socket(mServerAddress, mServerPort);
				// mSocket.setKeepAlive(true);
				if (mSocket != null) {
					// 获取输出流、输入流
					mOutputStream = mSocket.getOutputStream();
					mInputStream = mSocket.getInputStream();

					state = ProcessorStatus.INIT;

				}

			} catch (Exception e) {
				e.printStackTrace();
				CustomLog.e(tag, "connect fail " + e.toString());
				mHandler.sendEmptyMessageDelayed(MSG_SOCKET_CONNECTFAIL, 1000);
				// mHandler.sendEmptyMessage(MSG_SOCKET_CONNECTFAIL);
				return;
			}
			CustomLog.d(tag, "connect success");
			mHandler.sendEmptyMessage(MSG_SOCKET_CONNECT);
		}

	}

	class SocketReceiveThread extends Thread {
		private boolean threadExit;

		public SocketReceiveThread() {
			threadExit = false;
		}

		public void run() {
			byte[] buffer = new byte[1024];
			while (threadExit == false) {
				try {
					if (null == tcpReceiver) {
						CustomLog.w(tag, "no receiver handler, ignore data");
						continue;
					}
					// 读取数据，返回值表示读到的数据长度。-1表示结束
					int count = mInputStream.read(buffer);
					if (count == -1) {
						CustomLog.w(tag, "read read -1");
						mHandler.sendEmptyMessage(MSG_SOCKET_DISCONNECT);
						break;
					} else {
						String receiveData = new String(buffer, 0, count);
						CustomLog.d(tag, "read buffer:" + receiveData
								+ ",count=" + count);
						Message msg = new Message();
						msg.what = MSG_RECEIVE_DATA;
						msg.obj = receiveData;
						mHandler.sendMessage(msg);
					}
				} catch (IOException e) {
					e.printStackTrace();
					CustomLog.e(tag, "read fail " + e.toString());
				}
			}
		}

		void threadExit() {
			threadExit = true;
		}
	};
}
