package cn.redcdn.network.udp;

import java.net.DatagramPacket;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import cn.redcdn.log.CustomLog;

enum ProcessorStatus {
	INIT, DESTROY
}

public class UDPProcessor {
	final static String tag = UDPProcessor.class.getName();
	final static int CMD_BUFFER = 1;

	private String destIp = "127.0.0.1";

	private int destPort;

	private UDPSocket socket = new UDPSocket();

	private ProcessorStatus state = ProcessorStatus.DESTROY;

	UDPReceiver udpReceiver = null;
	WhiteBoardReceiver whiteBoardReceiver = null;

	class RecvThread extends Thread {
		@Override
		public void run() {
			super.run();
			while (ProcessorStatus.INIT == state) { // TODO 编译器是否回优化为常量
				DatagramPacket packet = socket.receive();
				if (null != packet) {
					if (null == udpReceiver && null == whiteBoardReceiver) {
						CustomLog.w(tag, "no receiver handler, ignore data");
						continue;
					}
					CustomLog.d(tag, "UDPProcessor receive datalength="
							+ packet.getLength());
					if (udpReceiver != null) {
						udpReceiver.process(new String(packet.getData(), 0,
								packet.getLength()));
					}

					if (whiteBoardReceiver != null) {
						byte[] data = new byte[packet.getLength()];
						System.arraycopy(packet.getData(), 0, data, 0,
								packet.getLength());

						whiteBoardReceiver.process(data);
					}

				} else {
					// try {
					// Thread.sleep(3000); //TODO 处理当循环接收异常出现cpu高的问题，需要验证；
					// } catch (InterruptedException e){
					// }
				}
			}

			CustomLog.d(tag, "UDPProcessor recv thread exit");
		}
	};

	class SendHandler extends Handler {
		public SendHandler(Looper looper) {
			super(looper);
		}

		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case CMD_BUFFER:
				if (ProcessorStatus.INIT != state) {
					CustomLog.e(tag, "UDPProcessor don't init");
					return;
				}
				if (udpReceiver != null) {
					byte[] sendData = ((String) msg.obj).getBytes();
					boolean result = socket.send(destIp, destPort, sendData);
					CustomLog.d(tag, "UDPProcessor send:" + (String) msg.obj
							+ ",result:" + result + " length:"
							+ sendData.length);
					break;
				}

				if (whiteBoardReceiver != null) {
					byte[] sendData = ((byte[]) msg.obj);
					boolean result = socket.send(destIp, destPort, sendData);
					CustomLog.d(tag, "UDPProcessor send:" + ",result:" + result
							+ " length:" + sendData.length);
					break;
				}
				break;

			default:
				break;
			}
		}
	};

	RecvThread recvThread = null;

	SendHandler sendHander = null;
	HandlerThread sendThread = null;

	public UDPProcessor(UDPReceiver receiver) {
		udpReceiver = receiver;
	}

	public UDPProcessor(WhiteBoardReceiver receiver) {
		whiteBoardReceiver = receiver;
	}

	public int init() {
		CustomLog.d(tag, "UDPProcessor init");

		if (ProcessorStatus.DESTROY != state) {
			CustomLog.e(tag, "UDPProcessor has initialized");
			return -1;
		}

		int localPort = socket.init();
		if (localPort <= 0) {
			CustomLog.e(tag, "UDPProcessor init udp socket failed");
			return -2;
		}

		state = ProcessorStatus.INIT;

		recvThread = new RecvThread();
		recvThread.start();

		sendThread = new HandlerThread("SendUDPThread");
		sendThread.start();
		sendHander = new SendHandler(sendThread.getLooper());

		return localPort;
	}

	public void destroy() {
		CustomLog.d(tag, "UDPProcessor destroy");

		if (ProcessorStatus.DESTROY == state) {
			CustomLog.e(tag, "UDPProcessor has destroyed");
			return;
		}

		state = ProcessorStatus.DESTROY;

		// 在 socket.release 之前发送一个空包，避免长时间等待返回
		// TODO 1. 不能在主线程发送
		// 2. 该send可能会发不出去
		Message msg = Message.obtain();
		msg.what = CMD_BUFFER;
		msg.obj = "quit";
		sendHander.sendMessage(msg);

		socket.release();

		try {
			recvThread.join();
		} catch (InterruptedException e) {
			CustomLog.d(tag, "UDPProcessor join thread failed");
		}

		recvThread = null;

		CustomLog.d(tag, "UDPProcessor send thread exit");
		sendThread.quit();
		sendThread = null;

		sendHander = null;
	}

	public int setDest(String destIp, int destPort) {
		if (null == destIp || destIp.isEmpty() || destPort <= 0) {
			return -1;
		}

		this.destIp = destIp;
		this.destPort = destPort;

		return 0;
	}

	public int setDest(int destPort) {
		return setDest("127.0.0.1", destPort);
	}

	public int send(String cmdBuffer) {
		CustomLog.d(tag, "UDPProcessor async send cmd:" + cmdBuffer);

		if (ProcessorStatus.INIT != state) {
			CustomLog.e(tag, "UDPProcessor don't init");
			return -1;
		}

		Message msg = Message.obtain();
		msg.what = CMD_BUFFER;
		msg.obj = cmdBuffer;
		sendHander.sendMessage(msg);

		return 0;
	}

	public int send(byte[] cmdBuffer) {
		CustomLog.d(tag, "UDPProcessor async send cmd,length:"
				+ cmdBuffer.length);

		if (ProcessorStatus.INIT != state) {
			CustomLog.e(tag, "UDPProcessor don't init");
			return -1;
		}

		Message msg = Message.obtain();
		msg.what = CMD_BUFFER;
		msg.obj = cmdBuffer;
		sendHander.sendMessage(msg);

		return 0;
	}
}
